import requests
import warnings
import pycountry
from geoip2fast import GeoIP2Fast

def get_ip() -> str:
	try:
		public_ip = requests.get('https://api.ipify.org').text
	except requests.RequestException:
		raise requests.ConnectionError("Cannot find public IP, manually determine a country.")
	return public_ip

def get_location() -> str:
	warnings.filterwarnings('ignore')
	geoip = GeoIP2Fast()
	try:
		found_name = geoip.lookup(get_ip()).country_name
		return convert_country_to_iso(found_name)
	except requests.RequestException:
		return "World"

def convert_country_to_iso(country_name: str):
	try:
		result = pycountry.countries.search_fuzzy(country_name)[0]
		return result.alpha_3
	except (LookupError, IndexError):
		return "World"


"""
A lightweight package to measure the ecological and financial effect of training and evaluation of pytorch projects.
"""

from .core import TrainTracker, EvalTracker, evaluate, train, Mode
from .datahandler import DataHandler
from ._geolocator import get_location, get_ip

__version__ = "0.2.5"
__author__ = "David Leonard Nagy"
__copyright__ = "Copyright 2026, David Leonard Nagy"
__credits__ = ["David Leonard Nagy"]
__license__ = "MIT"
__maintainer__ = "David Leonard Nagy"
__email__ = "nagy.david.leonard@gmail.com"
__status__ = "Development"

__all__ = [
    "TrainTracker",
    "EvalTracker",
    "DataHandler",
    "evaluate",
    "train",
    "Mode"
]


import pytest
from unittest.mock import patch, MagicMock
import requests
import ecotorch._geolocator as geolocator

def test_get_ip_success():
    with patch('requests.get') as mock_get:
        mock_get.return_value.text = '1.2.3.4'
        assert geolocator.get_ip() == '1.2.3.4'

def test_get_ip_failure():
    with patch('requests.get', side_effect=requests.RequestException):
        with pytest.raises(requests.ConnectionError):
            geolocator.get_ip()

def test_convert_country_to_iso_success():
    with patch('ecotorch._geolocator.pycountry.countries.search_fuzzy') as mock_search:
        mock_country = MagicMock()
        mock_country.alpha_3 = 'USA'
        mock_search.return_value = [mock_country]

        assert geolocator.convert_country_to_iso('United States') == 'USA'

def test_convert_country_to_iso_failure():
    with patch('ecotorch._geolocator.pycountry.countries.search_fuzzy', side_effect=LookupError):
        assert geolocator.convert_country_to_iso('Unknown Country') == 'World'

def test_get_location_success():
    with patch('ecotorch._geolocator.get_ip', return_value='1.2.3.4'), \
         patch('ecotorch._geolocator.GeoIP2Fast') as mock_geoip_cls, \
         patch('ecotorch._geolocator.convert_country_to_iso', return_value='USA') as mock_convert:

        mock_geoip_instance = mock_geoip_cls.return_value
        mock_geoip_instance.lookup.return_value.country_name = 'United States'

        assert geolocator.get_location() == 'USA'
        mock_convert.assert_called_with('United States')



def test_get_location_requests_failure():
    # If underlying IP or lookup fails with requests.RequestException, we fall back to 'World'
    with patch('ecotorch._geolocator.get_ip', side_effect=requests.RequestException):
        assert geolocator.get_location() == 'World'



import torch
import torch.nn as nn
import torch.nn.functional as f
import torchvision
import torchvision.transforms as transforms
import torch.optim as optim
import os

from ecotorch import evaluate, train, TrainTracker, EvalTracker

# Deletable, I mostly run my codes in Terminal, so it is cleaner from me to run the tests this way
os.system('clear' if os.name != "nt" else 'cls')


EPOCH = 40
BATCH_SIZE = 16
device = "cuda" if torch.cuda.is_available() else ("mps" if torch.mps.is_available() else "cpu")


transform = transforms.ToTensor()

train_dataset = torchvision.datasets.MNIST(
    root="./data",
    train=True,
    download=True,
    transform=transform
)

test_dataset = torchvision.datasets.MNIST(
    root="./data",
    train=True,
    download=True,
    transform=transform
)

train_loader = torch.utils.data.DataLoader(
    train_dataset,
    batch_size=BATCH_SIZE,
    shuffle=True,
    num_workers=0
)

test_loader = torch.utils.data.DataLoader(
    test_dataset,
    batch_size=BATCH_SIZE,
    shuffle=False,
    num_workers=0
)

class TestNet(nn.Module):
    def __init__(self):
        super().__init__()
        self.flatten = nn.Flatten()
        self.fc1 = nn.Linear(784, 32)
        self.fc2 = nn.Linear(32, 32)
        self.fc3 = nn.Linear(32, 10)

    def forward(self, x):
        x = self.flatten(x)
        x = f.relu(self.fc1(x))
        x = f.relu(self.fc2(x))
        x = self.fc3(x)
        return x

net = TestNet().to(device)
criterion = nn.CrossEntropyLoss().to(device)
optimizer = optim.SGD(net.parameters(), lr=0.001, momentum=0.9)

with TrainTracker(epochs=EPOCH, model=net, train_dataloader=train_loader) as train_tracker:
    net, final_loss, first_loss, _, _ = train(net, criterion, optimizer, train_loader, EPOCH, device)
print(train_tracker.calculate_efficiency_score(initial_loss=first_loss, final_loss=final_loss))

print("Finished training")

with EvalTracker(test_dataloader=test_loader, train_tracker=train_tracker) as eval_tracker:
    acc = evaluate(net, test_loader, device)

print(eval_tracker.calculate_efficiency_score(accuracy=acc))

print(f"Accuracy: {acc*100}%")